require "em-websocket"
module Rex
  module Server
    class WebsocketServer
      def initialize(message_broker, outbox, host: "0.0.0.0", port: 8080)
        @message_broker = message_broker
        @outbox = outbox
        @host = host
        @port = port
      end

      def start
        EM::WebSocket.run(host: @host, port: @port) do |connection|
          setup_connection_handling(connection)
        end
      end

      def setup_connection_handling(connection)
        connection.onopen do
          connection_id = @message_broker.register(connection)

          connection.onmessage do |message|
            @outbox.push([connection_id, message])
          end

          connection.onclose do
            @message_broker.unregister(connection_id)
          end
        end
      end
    end
  end
end
